@@ -52,10 +52,9 @@
 #undef USE_OPML
 
 #ifdef DEBUG
-static pthread_mutex_t debugprint_mutex = PTHREAD_MUTEX_INITIALIZER;
-#define DEBUG_PRINT(...) pthread_mutex_lock(&debugprint_mutex); \
-                         g_message(__VA_ARGS__); \
-                         pthread_mutex_unlock(&debugprint_mutex);
+#define DEBUG_PRINT(...) printf("%s(%u): ",__func__,__LINE__); \
+                         printf(__VA_ARGS__);printf("\n");
+//#define DEBUG_PRINT(...)
 #else
 #define DEBUG_PRINT(...)
 #endif
@@ -97,8 +96,16 @@ static pthread_mutex_t getfeed_mutex = PTHREAD_MUTEX_INITIALIZER;
 /* $B%a%C%;!<%8%@%$%"%m%0I=<(MQ$N%m%C%/(B */
 static pthread_mutex_t msgdisp_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-#define NEWSREADER_LOCK()	pthread_mutex_lock(&newsreader_mutex)
-#define NEWSREADER_UNLOCK()	pthread_mutex_unlock(&newsreader_mutex)
+//#define NEWSREADER_LOCK()	pthread_mutex_lock(&newsreader_mutex)
+//#define NEWSREADER_UNLOCK()	pthread_mutex_unlock(&newsreader_mutex)
+
+#ifdef DEBUG
+#define NEWSREADER_LOCK() printf("lock %s: %u\n",__func__,__LINE__);XFCE_PANEL_LOCK();
+#define NEWSREADER_UNLOCK() printf("unlock %s: %u\n",__func__,__LINE__);XFCE_PANEL_UNLOCK();
+#else
+#define NEWSREADER_LOCK() XFCE_PANEL_LOCK();
+#define NEWSREADER_UNLOCK() XFCE_PANEL_UNLOCK();
+#endif
 
 const gint INIT_LABEL_WIDTH = 300;
 const gint INIT_UPDATE_MIN = 60;
@@ -120,6 +127,8 @@ copy_feeddata(feeddata_t* pFeedData)
 	if(!pFeedData)
 		return NULL;
 
+	DEBUG_PRINT("copy_feeddata start");
+
 	pNewFeedData = create_new_feeddata();
 	if(pFeedData->pchURI)
 		pNewFeedData->pchURI = g_strdup(pFeedData->pchURI);
@@ -128,7 +137,8 @@ copy_feeddata(feeddata_t* pFeedData)
 	pNewFeedData->tmLastModified = pFeedData->tmLastModified;
 	pNewFeedData->pIcon = pFeedData->pIcon;
 	pNewFeedData->iValidHours = pFeedData->iValidHours;
-	
+	DEBUG_PRINT("copy_feeddata end");
+
 	return pNewFeedData;
 }
 
@@ -496,13 +506,15 @@ message_register(newsreader_t* newsreader, const gchar* pchMsg)
 
 /* $B%]%C%W%"%C%W%a%K%e!<MQ%j%9%H$N%/%j%"(B */
 static void
-ClearFeedListMenu(newsreader_t* newsreader)
+clear_feed_list_menu(newsreader_t* newsreader)
 {
 	GtkWidget *menuFeedList;
 
-	if(!(menuFeedList = gtk_menu_item_get_submenu(newsreader->wItmFeedList)))
-		return;
 
+	if(!(menuFeedList = gtk_menu_item_get_submenu(newsreader->wItmFeedList))){
+		return;
+	}
+	
 	/* $B3+J|$O$3$l$G$h$$(B? */
 	gtk_widget_destroy(menuFeedList);
 
@@ -510,16 +522,19 @@ ClearFeedListMenu(newsreader_t* newsreader)
 	g_list_free(newsreader->poMenuItemList);
 	newsreader->poMenuItemList = NULL;
 
+
 	return;
 }
 
 /* $B%K%e!<%90lMw%a%K%e!<%/%j%C%/;~$N=hM}(B */
-static gboolean
+static void
 on_itemmenu_activate_cb(GtkMenuItem *item, gpointer data)
 {
 	newsreader_t* newsreader = (newsreader_t*)data;
 	GList* poList;
 	itemuri_t* pItem;
+	
+	NEWSREADER_LOCK();
 
 	for(poList = newsreader->poMenuItemList; poList; poList = poList->next){
 		if((pItem = (itemuri_t*)poList->data) &&
@@ -527,12 +542,12 @@ on_itemmenu_activate_cb(GtkMenuItem *item, gpointer data)
 			break;
 	}
 	
-	if(!poList)
-		return FALSE;
+	if(poList)
+		notify_uri(newsreader->pchBrowserCmd, pItem->pchURI);	
+		
+	NEWSREADER_UNLOCK();
 
-	notify_uri(newsreader->pchBrowserCmd, pItem->pchURI);	
-
-	return TRUE;
+	return;
 
 }
 
@@ -566,7 +581,7 @@ getFaviconPixbuf(newsreader_t* newsreader, const gchar* pchUri)
 
 /* $B%]%C%W%"%C%W%a%K%e!<$N99?7(B */
 static int
-UpdateFeedListMenu(newsreader_t* newsreader)
+update_feed_list_menu(newsreader_t* newsreader)
 {
 	GList *pFeedList, *pItemList;
 	feeddata_t* pFeedData;
@@ -582,7 +597,7 @@ UpdateFeedListMenu(newsreader_t* newsreader)
 	GtkWidget* itmItem;
 	int iFeedNo, iItemNo;
 
-	ClearFeedListMenu(newsreader);
+	clear_feed_list_menu(newsreader);
 
 	menuFeedList = gtk_menu_new();
 
@@ -642,6 +657,11 @@ UpdateFeedListMenu(newsreader_t* newsreader)
 					continue;
 				}
 				
+				if(!pItem->title){
+					DEBUG_PRINT("title is null");
+					continue;
+				}
+				
 				strLabel = g_string_new(pItem->title->str);
 				if(pItem->tmModified >= 0){
 					t_st = localtime(&pItem->tmModified);
@@ -693,6 +713,7 @@ UpdateFeedListMenu(newsreader_t* newsreader)
 
 	DEBUG_PRINT("UpdateFeedListMenu End");
 
+
 	return iFeedNo;
 }
 
@@ -974,6 +995,8 @@ GetFeedList(newsreader_t* newsreader)
 		if(!poOutline || !poOutline->bAvailable)
 			continue;
 
+		NEWSREADER_LOCK();
+
 		/* $BA02s<hF@$N(BFeedDataList$B$+$iL\E*$N(BFeed$B$r8!:w(B */
 		for(poFeedList = newsreader->g_pFeedList; poFeedList; poFeedList = poFeedList->next){
 			if(strcmp(((feeddata_t*)(poFeedList->data))->pchURI, poOutline->strURL->str) == 0){
@@ -981,11 +1004,16 @@ GetFeedList(newsreader_t* newsreader)
 			}
 		}
 		
+		if(poFeedList)
+			pFeedData = copy_feeddata((feeddata_t*)poFeedList->data);
+
+		NEWSREADER_UNLOCK();
+		
 		DEBUG_PRINT("search feedlist end");
 
 		/* $BA02s<hF@$N(BFeedData$B$,$"$C$?(B */
 		if(poFeedList){
-			pFeedData = copy_feeddata((feeddata_t*)poFeedList->data);
+			DEBUG_PRINT("FeedData found");
 			if((bufflen = httpfetch(pFeedData->pchURI, &buff, pFeedData->tmLastModified, &tmRetLastModified)) < 0){
 				DEBUG_PRINT("httpfetch faild");
 				if(HF_NOT_MODIFIED == bufflen ){
@@ -1013,6 +1041,7 @@ GetFeedList(newsreader_t* newsreader)
 			
 		}
 		else{
+			DEBUG_PRINT("FeedData not found");
 			if((bufflen = httpfetch(poOutline->strURL->str, &buff, -1, &tmRetLastModified)) < 0){
 				DEBUG_PRINT("httpfetch faild");
 				if(!(buff = get_cache_file(poOutline->strURL->str, &bufflen, newsreader->pCacheTable))){
@@ -1021,6 +1050,7 @@ GetFeedList(newsreader_t* newsreader)
 				}
 			}
 			else{
+				DEBUG_PRINT("write_cache_file");
 				write_cache_file(buff, bufflen, poOutline->strURL->str, newsreader->pCacheTable);
 			}
 			
@@ -1076,7 +1106,7 @@ GetFeedList(newsreader_t* newsreader)
 	}
 	g_list_free(poOldList);
 
-	UpdateFeedListMenu(newsreader);
+	update_feed_list_menu(newsreader);
 
 	NEWSREADER_UNLOCK();
 
@@ -1092,9 +1122,10 @@ static gboolean SetUpdateFeedTimer(void *p_newsreader)
   struct newsreader_t *newsreader = (newsreader_t*) p_newsreader;
   pthread_t thread;
 
-  DEBUG_PRINT("SetUpdateFeedTimer start");
   NEWSREADER_LOCK();
 
+  DEBUG_PRINT("SetUpdateFeedTimer start");
+
   if (newsreader->iFeedTimerID) {
 	g_source_remove (newsreader->iFeedTimerID);
 	newsreader->iFeedTimerID = 0;
@@ -1107,6 +1138,7 @@ static gboolean SetUpdateFeedTimer(void *p_newsreader)
 												  (GtkFunction) SetUpdateFeedTimer, newsreader);
 
   NEWSREADER_UNLOCK();
+
   DEBUG_PRINT("SetUpdateFeedTimer end");
 
   return (1);
@@ -1467,17 +1499,17 @@ newsreader_attach_callback (Control * control, const char *signal,
     g_signal_connect (newsreader->wBtnMenu, signal, callback, data);
 }
 
-static gboolean
+static void
 on_itemupdate_activate_cb(GtkMenuItem *item, gpointer data)
 {
 	newsreader_t *newsreader;
 	newsreader = (newsreader_t*) data;
 
 	SetUpdateFeedTimer( newsreader );
-	return TRUE;
+	return;
 }
 
-static gboolean
+static void
 on_menubutton_press_cb (newsreader_t *newsreader, GdkEvent *event)
 {
 	GtkMenu *menu;
@@ -1485,8 +1517,11 @@ on_menubutton_press_cb (newsreader_t *newsreader, GdkEvent *event)
 
 	//	g_return_val_if_fail (widget != NULL, FALSE);
 	//	g_return_val_if_fail (GTK_IS_MENU (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
+	g_return_if_fail (event != NULL);
 
+	NEWSREADER_LOCK();
+
+	DEBUG_PRINT("menu popup start");
 	/* The "widget" is the menu that was supplied when 
 	 * g_signal_connect_swapped() was called.
 	 */
@@ -1497,11 +1532,13 @@ on_menubutton_press_cb (newsreader_t *newsreader, GdkEvent *event)
 		if (event_button->button == 1){
 			gtk_menu_popup (menu, NULL, NULL, NULL, NULL, 
 							event_button->button, event_button->time);
-			return TRUE;
 		}
 	}
 
-	return FALSE;
+	DEBUG_PRINT("menu popup end");
+
+	NEWSREADER_UNLOCK();
+	return;
 }
 
 /* $B%\%?%s%j%j!<%9$G8F$P$l$F%V%i%&%6$r5/F0$9$k(B */
@@ -1534,6 +1571,9 @@ on_button_release_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
 static gboolean
 on_enter_notify_cb (GtkEventBox *widget, GdkEventCrossing *event, newsreader_t* newsreader)
 {
+
+	NEWSREADER_LOCK();
+
 	/* $B%^%&%9%+!<%=%k$NJQ99(B */
 	gdk_window_set_cursor ( GTK_WIDGET(newsreader->wEvItem)->window, gdk_cursor_new ( GDK_HAND2));
 
@@ -1548,6 +1588,8 @@ on_enter_notify_cb (GtkEventBox *widget, GdkEventCrossing *event, newsreader_t*
 		gtk_tooltips_enable(newsreader->wTooltips);
 	}
 
+	NEWSREADER_UNLOCK();
+
 	return TRUE;  
 }
 
@@ -2214,7 +2256,7 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 	newsreader->g_pCurFeed = NULL;
 	newsreader->g_pCurItem = NULL;
 
-	UpdateFeedListMenu(newsreader);
+	update_feed_list_menu(newsreader);
 
 	g_free(newsreader->pchURI);
 
