@@ -85,6 +85,7 @@ item_t* create_new_item()
 {
 	item_t* pItem = NULL;
 	pItem = g_new0(item_t, 1);
+
 	pItem->tmModified = -1;
 	pItem->bIsNew = FALSE;
 
@@ -99,23 +100,35 @@ copy_items(GList* pItemList)
 	GList* pNewList = NULL;
 	GList* pList;
 	
-	if(!pItemList)
+	if(!pItemList){
+		g_warning("copy_items faild");
 		return NULL;
-
+	}
+	
 	for(pList = pItemList; pList; pList = pList->next){
 		pItem = (item_t*)pList->data;
-		if(!pItem)
+		if(!pItem){
 			continue;
+		}
+		
+		printf("pItem = %p\n", pItem);
+
 		pNewItem = create_new_item();
-		if(pItem->title)
+		if(pItem->title){
 			pNewItem->title = g_string_new(pItem->title->str);
-		if(pItem->url)
+		}
+		
+		if(pItem->url){		
 			pNewItem->url = g_string_new(pItem->url->str);
+			g_warning("%s", pItem->url->str);
+		}
+		
 		pNewItem->tmModified = pItem->tmModified;
 		pNewItem->bIsNew = pItem->bIsNew;
 
 		pNewList = g_list_append(pNewList, pNewItem);
 	}	
+
 	return pNewList;
 }
 
@@ -124,9 +137,11 @@ copy_feed(feed_t* pFeed)
 {
 	feed_t* pNewFeed = NULL;
 
-	if(!pFeed)
+	if(!pFeed){
+		g_warning("copy_feed faild");
 		return NULL;
-
+	}
+	
 	pNewFeed = create_new_feed();
 	if(pFeed->title)
 		pNewFeed->title = g_string_new(pFeed->title->str);
@@ -166,7 +181,8 @@ free_item(item_t* pitem)
 
 	if(pitem->url)
 		g_string_free(pitem->url, TRUE);
-	free(pitem);
+
+	g_free(pitem);
 	return;
 }
 
@@ -189,7 +205,7 @@ free_feed(feed_t* pfeed)
 	if(pfeed->url)
 		g_string_free(pfeed->url, TRUE);
 
-	free(pfeed);
+	g_free(pfeed);
   
 	return;
 }
@@ -298,7 +314,7 @@ get_channel(feed_t* pfeed, xmlDocPtr doc, xmlNode* node)
 static gboolean
 get_rss10_item(feed_t* pfeed, xmlDocPtr doc, xmlNode* node)
 {
-	xmlNode *cur = NULL;
+	xmlNodePtr cur = NULL;
 	item_t* pitem = create_new_item();
 
 	for (cur = node; cur != NULL; cur = cur->next) {
@@ -323,7 +339,8 @@ get_rss10_item(feed_t* pfeed, xmlDocPtr doc, xmlNode* node)
 	}
 	else{
 		pfeed->items = g_list_append(pfeed->items, pitem);
-	}	
+	}
+
 	return TRUE;
 }
 
